Package: grImport2
Version: 0.3-3
Depends: R (>= 2.12.0)
Imports: methods, grDevices, grid, XML, png, jpeg, base64enc
Suggests: gridSVG, rsvg
Title: Importing 'SVG' Graphics
Authors@R: c(person("Simon", "Potter", role = c("aut"),
                    email = "simon.potter@auckland.ac.nz"),
             person("Paul", "Murrell", role = c("aut", "cre"),
                    email = "paul@stat.auckland.ac.nz",
                    comment = c(ORCID = "0000-0002-3224-8858")))
Description: Functions for importing external vector images and
  drawing them as part of 'R' plots.  This package is different from the
  'grImport' package because, where that package imports 'PostScript'
  format images, this package imports 'SVG' format images.  Furthermore,
  this package imports a specific subset of 'SVG', so external images
  must be preprocessed using a package like 'rsvg' to produce 'SVG'
  that this package can import.  'SVG' features that are not supported
  by 'R' graphics, e.g., gradient fills, can be imported and then
  exported via the 'gridSVG' package.
URL:
        https://r-forge.r-project.org/projects/grimport/,https://stattech.wordpress.fos.auckland.ac.nz/2013/09/,https://stattech.wordpress.fos.auckland.ac.nz/2016/08/30/2016-11-the-butterfly-affectation/,https://stattech.wordpress.fos.auckland.ac.nz/2019/11/25/2019-02-svg-in-svg-out/
License: GPL (>= 2)
NeedsCompilation: no
Packaged: 2024-07-30 07:55:29 UTC; pmur002
Author: Simon Potter [aut],
  Paul Murrell [aut, cre] (<https://orcid.org/0000-0002-3224-8858>)
Maintainer: Paul Murrell <paul@stat.auckland.ac.nz>
Repository: CRAN
Date/Publication: 2024-07-30 08:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 16:07:21 UTC; unix
