import(grid)
import(XML)
import(methods)
importFrom(grDevices, pdf, dev.cur, dev.off, dev.set, rgb)
importFrom(base64enc, base64decode)
importFrom(png, readPNG)
importFrom(jpeg, readJPEG)

exportClasses("Picture")
exportClasses("PictureClipPath")
exportClasses("PictureContent")
exportClasses("PictureDefinitions")
exportClasses("PictureFeColorMatrix")
exportClasses("PictureFilter")
exportClasses("PictureGradientStop")
exportClasses("PictureGroup")
exportClasses("PictureImage")
exportClasses("PictureLinearGradient")
exportClasses("PictureMask")
exportClasses("PicturePath")
    exportClasses("PathData")
        exportClasses("PathSegment")
        exportClasses("PathMoveTo")
        exportClasses("PathLineTo")
        exportClasses("PathCurveTo")
        exportClasses("PathClosePath")
exportClasses("PicturePattern")
exportClasses("PicturePath")
exportClasses("PictureRadialGradient")
exportClasses("PictureRect")
exportClasses("PictureSummary")
exportClasses("PictureSymbol")

exportMethods("[")
exportMethods("[[") 
exportMethods("applyTransform")
exportMethods("grobify")
exportMethods("getDef")
exportMethods("setDef")

# Just S4 wrappers
#exportClasses("gpar")
#exportClasses("nativeRaster")

# Path wrappers
export(moveTo, lineTo, curveTo, closePath)

export(readPicture)
export(grid.picture)
export(pictureGrob)
export(grid.symbols)
export(symbolsGrob)

S3method("makeContent", "PictureGrob")
S3method("makeContext", "picRect")
S3method("makeContent", "picRect")
S3method("makeContent", "picPolyline")
S3method("makeContent", "picPath")
S3method("makeContext", "picComplexPath")
